@@ -1,4 +1,13 @@
 {-# LANGUAGE CPP, PatternGuards, Rank2Types #-}
+-- This OPTIONS_GHC line is a workaround for
+-- https://gitlab.haskell.org/ghc/ghc/-/issues/18320, a bug which only occurs
+-- when running specific TemplateHaskell code while both profiling and
+-- optimisations are enabled. The code in this file triggers the bug, so until
+-- it is fixed, we work around the issue by disabling optimisations in this
+-- file. The code in this file only runs at compile-time, the code _generated_
+-- by makeBaseFunctor will still get optimized if the file which calls
+-- makeBaseFunctor is optimized.
+{-# OPTIONS_GHC -O0 #-}
 module Data.Functor.Foldable.TH
   ( MakeBaseFunctor(..)
   , BaseRules
@@ -220,17 +229,22 @@ makePrimForDI rules mkInstance'
                 , datatypeInstTypes = instTys
                 , datatypeCons      = cons
                 , datatypeVariant   = variant }) = do
-    when isDataFamInstance $
-      fail "makeBaseFunctor: Data families are currently not supported."
+    checkAllowed
     makePrimForDI' rules mkInstance'
                    (variant == Newtype) tyName
                    (map toTyVarBndr instTys) cons
   where
-    isDataFamInstance = case variant of
-                          DataInstance    -> True
-                          NewtypeInstance -> True
-                          Datatype        -> False
-                          Newtype         -> False
+    checkAllowed =
+      case variant of
+        Datatype        -> pure ()
+        Newtype         -> pure ()
+        DataInstance    -> dataFamilyError
+        NewtypeInstance -> dataFamilyError
+#if MIN_VERSION_th_abstraction(0,5,0)
+        TH.Abs.TypeData -> fail "makeBaseFunctor: `type data` declarations are not supported."
+#endif
+
+    dataFamilyError = fail "makeBaseFunctor: Data families are currently not supported."
 
     toTyVarBndr :: Type -> TyVarBndrUnit
     toTyVarBndr (VarT n)          = plainTV n
